@@ -61,8 +61,8 @@
    #include <botan/hmac_drbg.h>
 #endif
 
-#if defined(BOTAN_HAS_RDRAND_RNG)
-   #include <botan/rdrand_rng.h>
+#if defined(BOTAN_HAS_PROCESSOR_RNG)
+   #include <botan/processor_rng.h>
 #endif
 
 #if defined(BOTAN_HAS_CHACHA_RNG)
@@ -136,6 +136,10 @@
    #include <botan/passhash9.h>
 #endif
 
+#if defined(BOTAN_HAS_ZFEC)
+  #include <botan/zfec.h>
+#endif
+
 namespace Botan_CLI {
 
 using Botan::Timer;
@@ -479,8 +483,6 @@ class Speed final : public Command
 
          const std::vector<size_t> buf_sizes = unique_buffer_sizes(get_arg("buf-size"));
 
-         Botan::CPUID::initialize();
-
          for(std::string cpuid_to_clear : Botan::split_on(get_arg("clear-cpuid"), ','))
             {
             auto bits = Botan::CPUID::bit_from_string(cpuid_to_clear);
@@ -676,6 +678,12 @@ class Speed final : public Command
                bench_passhash9();
                }
 #endif
+#if defined(BOTAN_HAS_ZFEC)
+            else if(algo == "zfec")
+               {
+               bench_zfec(msec);
+               }
+#endif
 #if defined(BOTAN_HAS_POLY_DBL)
             else if(algo == "poly_dbl")
                {
@@ -771,11 +779,11 @@ class Speed final : public Command
                bench_rng(Botan::system_rng(), "System_RNG", msec, buf_sizes);
 #endif
 
-#if defined(BOTAN_HAS_RDRAND_RNG)
-               if(Botan::CPUID::has_rdrand())
+#if defined(BOTAN_HAS_PROCESSOR_RNG)
+               if(Botan::Processor_RNG::available())
                   {
-                  Botan::RDRAND_RNG rdrand;
-                  bench_rng(rdrand, "RDRAND", msec, buf_sizes);
+                  Botan::Processor_RNG hwrng;
+                  bench_rng(hwrng, "Processor_RNG", msec, buf_sizes);
                   }
 #endif
 
@@ -1128,15 +1136,15 @@ class Speed final : public Command
          {
          for(std::string group_name : groups)
             {
-            const Botan::EC_Group group(group_name);
+            const Botan::EC_Group ec_group(group_name);
 
             std::unique_ptr<Timer> add_timer = make_timer(group_name + " add");
             std::unique_ptr<Timer> addf_timer = make_timer(group_name + " addf");
             std::unique_ptr<Timer> dbl_timer = make_timer(group_name + " dbl");
 
-            const Botan::PointGFp& base_point = group.get_base_point();
-            Botan::PointGFp non_affine_pt = group.get_base_point() * 1776; // create a non-affine point
-            Botan::PointGFp pt = group.get_base_point();
+            const Botan::PointGFp& base_point = ec_group.get_base_point();
+            Botan::PointGFp non_affine_pt = ec_group.get_base_point() * 1776; // create a non-affine point
+            Botan::PointGFp pt = ec_group.get_base_point();
 
             std::vector<Botan::BigInt> ws(Botan::PointGFp::WORKSPACE_SIZE);
 
@@ -1173,13 +1181,13 @@ class Speed final : public Command
          {
          for(std::string group_name : groups)
             {
-            const Botan::EC_Group group(group_name);
+            const Botan::EC_Group ec_group(group_name);
 
             std::unique_ptr<Timer> mult_timer = make_timer(group_name + " Montgomery ladder");
             std::unique_ptr<Timer> blinded_mult_timer = make_timer(group_name + " blinded comb");
             std::unique_ptr<Timer> blinded_var_mult_timer = make_timer(group_name + " blinded window");
 
-            const Botan::PointGFp& base_point = group.get_base_point();
+            const Botan::PointGFp& base_point = ec_group.get_base_point();
 
             std::vector<Botan::BigInt> ws;
 
@@ -1187,15 +1195,15 @@ class Speed final : public Command
                   blinded_mult_timer->under(runtime) &&
                   blinded_var_mult_timer->under(runtime))
                {
-               const Botan::BigInt scalar(rng(), group.get_p_bits());
+               const Botan::BigInt scalar(rng(), ec_group.get_p_bits());
 
                const Botan::PointGFp r1 = mult_timer->run([&]() { return base_point * scalar; });
 
                const Botan::PointGFp r2 = blinded_mult_timer->run(
-                  [&]() { return group.blinded_base_point_multiply(scalar, rng(), ws); });
+                  [&]() { return ec_group.blinded_base_point_multiply(scalar, rng(), ws); });
 
                const Botan::PointGFp r3 = blinded_var_mult_timer->run(
-                  [&]() { return group.blinded_var_point_multiply(base_point, scalar, rng(), ws); });
+                  [&]() { return ec_group.blinded_var_point_multiply(base_point, scalar, rng(), ws); });
 
                BOTAN_ASSERT_EQUAL(r1, r2, "Same point computed by Montgomery and comb");
                BOTAN_ASSERT_EQUAL(r1, r3, "Same point computed by Montgomery and window");
@@ -1214,17 +1222,17 @@ class Speed final : public Command
 
          for(std::string group_name : groups)
             {
-            const Botan::EC_Group group(group_name);
+            const Botan::EC_Group ec_group(group_name);
 
             while(uncmp_timer->under(runtime) && cmp_timer->under(runtime))
                {
                const Botan::BigInt k(rng(), 256);
-               const Botan::PointGFp p = group.get_base_point() * k;
+               const Botan::PointGFp p = ec_group.get_base_point() * k;
                const std::vector<uint8_t> os_cmp = p.encode(Botan::PointGFp::COMPRESSED);
                const std::vector<uint8_t> os_uncmp = p.encode(Botan::PointGFp::UNCOMPRESSED);
 
-               uncmp_timer->run([&]() { group.OS2ECP(os_uncmp); });
-               cmp_timer->run([&]() { group.OS2ECP(os_cmp); });
+               uncmp_timer->run([&]() { ec_group.OS2ECP(os_uncmp); });
+               cmp_timer->run([&]() { ec_group.OS2ECP(os_cmp); });
                }
 
             record_result(uncmp_timer);
@@ -1358,7 +1366,7 @@ class Speed final : public Command
                y.randomize(rng(), q_bits);
 
                div_timer->start();
-               Botan::divide(x, y, q1, r1);
+               Botan::vartime_divide(x, y, q1, r1);
                div_timer->stop();
 
                ct_div_timer->start();
@@ -1397,7 +1405,7 @@ class Speed final : public Command
                x.randomize(rng(), n_bits);
 
                div_timer->start();
-               Botan::divide(x, ten, q1, r1);
+               Botan::vartime_divide(x, ten, q1, r1);
                div_timer->stop();
 
                ct_div_timer->start();
@@ -1672,9 +1680,9 @@ class Speed final : public Command
 
             if(dec_timer->under(msec))
                {
-               auto dec_pt = dec_timer->run([&]() { return dec.decrypt(ciphertext); });
+               const auto dec_pt = dec_timer->run([&]() { return dec.decrypt(ciphertext); });
 
-               if(dec_pt != plaintext) // sanity check
+               if(!(dec_pt == plaintext)) // sanity check
                   {
                   error_output() << "Bad roundtrip in PK encrypt/decrypt bench\n";
                   }
@@ -2154,6 +2162,77 @@ class Speed final : public Command
          }
 #endif
 
+#if defined(BOTAN_HAS_ZFEC)
+      void bench_zfec(std::chrono::milliseconds msec)
+         {
+         const size_t k = 4;
+         const size_t n = 16;
+
+         Botan::ZFEC zfec(k, n);
+
+         const size_t share_size = 256 * 1024;
+
+         std::vector<uint8_t> input(share_size * k);
+         rng().randomize(input.data(), input.size());
+
+         std::vector<uint8_t> output(share_size * n);
+
+         auto enc_fn = [&](size_t share, const uint8_t buf[], size_t len)
+            {
+            std::memcpy(&output[share*share_size], buf, len);
+            };
+
+         auto enc_timer = make_timer("zfec " +
+                                     std::to_string(k) + "/" +
+                                     std::to_string(n),
+                                     input.size(),
+                                     "encode", "", input.size());
+
+         enc_timer->run_until_elapsed(msec, [&]() {
+            zfec.encode(input.data(), input.size(), enc_fn);
+         });
+
+         record_result(enc_timer);
+
+         auto dec_timer = make_timer("zfec " +
+                                     std::to_string(k) + "/" +
+                                     std::to_string(n),
+                                     input.size(),
+                                     "decode", "", input.size());
+
+         std::map<size_t, const uint8_t*> shares;
+         for(size_t i = 0; i != n; ++i)
+            {
+            shares[i] = &output[share_size * i];
+            }
+
+         // remove data shares to make decoding maximally expensive:
+         while(shares.size() != k)
+            {
+            shares.erase(shares.begin());
+            }
+
+         std::vector<uint8_t> recovered(share_size * k);
+
+         auto dec_fn = [&](size_t share, const uint8_t buf[], size_t len)
+            {
+            std::memcpy(&recovered[share * share_size], buf, len);
+            };
+
+         dec_timer->run_until_elapsed(msec, [&]() {
+            zfec.decode_shares(shares, share_size, dec_fn);
+         });
+
+         record_result(dec_timer);
+
+         if(recovered != input)
+            {
+            error_output() << "ZFEC recovery failed\n";
+            }
+         }
+
+#endif
+
 #if defined(BOTAN_HAS_POLY_DBL)
       void bench_poly_dbl(std::chrono::milliseconds msec)
          {
