@@ -480,9 +480,11 @@ std::map<std::string, std::function<Test* ()>>& Test::global_registry()
    }
 
 //static
-void Test::register_test(const std::string& name,
+void Test::register_test(const std::string& category,
+                         const std::string& name,
                          std::function<Test* ()> maker_fn)
    {
+   BOTAN_UNUSED(category);
    if(Test::global_registry().count(name) != 0)
       throw Test_Error("Duplicate registration of test '" + name + "'");
 
@@ -595,14 +597,6 @@ void Test::set_test_options(const Test_Options& opts)
 //static
 void Test::set_test_rng(std::unique_ptr<Botan::RandomNumberGenerator> rng)
    {
-#if defined(BOTAN_TARGET_OS_HAS_THREADS)
-   if(m_opts.test_threads() != 1)
-      {
-      m_test_rng.reset(new Botan::Serialized_RNG(rng.release()));
-      return;
-      }
-#endif
-
    m_test_rng.reset(rng.release());
    }
 
